from typing import Any, Dict, NoReturn

from chaosplt_auth.storage.interface import BaseAuthStorage

__all__ = ["MyAuthStorage"]


class MyAuthStorage(BaseAuthStorage):
    def __init__(self, config: Dict[str, Any]):
        self.some_flag = True

    def release(self) -> NoReturn:
        self.some_flag = False



from typing import Dict
import uuid
from uuid import UUID

import cherrypy
from flask import Flask
from werkzeug.test import Client

from chaosplt_auth.model import AccessToken, User
from chaosplt_auth.schemas import AccessTokenSchema, NewAccessTokenSchema, \
    token_schema, new_token_schema


def test_new_invalid_request_payload(api: Flask, api_client: Client,
                                     access_token_request: Dict[str, str],
                                     request_token: str):
    r = api_client.post(
        "/api/v1/auth/tokens",
        headers={
            "Accept": "application/json",
            "Authorization": "Bearer {}".format(request_token),
            "Content-Type": "application/json"
        },
        json={}
    )
    assert r.status_code == 422

    msg = r.json
    assert "name" in msg
    assert "user_id" in msg


def test_new_valid_request_payload(api: Flask, api_client: Client,
                                   access_token_request: Dict[str, str],
                                   request_token: str, account_service,
                                   user: User):
    account_service.registration.get.return_value = user
    r = api_client.post(
        "/api/v1/auth/tokens",
        headers={
            "Accept": "application/json",
            "Content-Type": "application/json",
            "Authorization": "Bearer {}".format(request_token),
        },
        json=access_token_request
    )
    assert r.status_code == 201


def test_cannot_create_for_other_user(api: Flask, api_client: Client,
                                   access_token_request: Dict[str, str],
                                   request_token: str, account_service,
                                   user: User):
    account_service.registration.get.return_value = user
    
    data = access_token_request.copy()
    data["user_id"] = str(uuid.uuid4())

    r = api_client.post(
        "/api/v1/auth/tokens",
        headers={
            "Accept": "application/json",
            "Content-Type": "application/json",
            "Authorization": "Bearer {}".format(request_token),
        },
        json=data
    )
    assert r.status_code == 422
    assert r.json["message"] == "Cannot create a token for another user"


def test_get_access_token(api: Flask, api_client: Client,
                          access_token_request: Dict[str, str],
                          request_token: str, account_service,
                          user: User):
    account_service.registration.get.return_value = user
    r = api_client.post(
        "/api/v1/auth/tokens",
        headers={
            "Accept": "application/json",
            "Content-Type": "application/json",
            "Authorization": "Bearer {}".format(request_token),
        },
        json=access_token_request
    )
    assert r.status_code == 201
    token = r.json

    r = api_client.get(
        "/api/v1/auth/tokens/{}".format(token["id"]),
        headers={
            "Accept": "application/json",
            "Authorization": "Bearer {}".format(request_token),
        }
    )
    assert r.status_code == 200


def test_delete_access_token(api: Flask, api_client: Client,
                          access_token_request: Dict[str, str],
                          request_token: str, account_service,
                          user: User):
    account_service.registration.get.return_value = user
    r = api_client.post(
        "/api/v1/auth/tokens",
        headers={
            "Accept": "application/json",
            "Content-Type": "application/json",
            "Authorization": "Bearer {}".format(request_token),
        },
        json=access_token_request
    )
    assert r.status_code == 201
    token = r.json

    r = api_client.get(
        "/api/v1/auth/tokens/{}".format(token["id"]),
        headers={
            "Accept": "application/json",
            "Authorization": "Bearer {}".format(request_token),
        }
    )
    assert r.status_code == 200

    r = api_client.delete(
        "/api/v1/auth/tokens/{}".format(token["id"]),
        headers={
            "Accept": "application/json",
            "Authorization": "Bearer {}".format(request_token),
        }
    )
    assert r.status_code == 204

    r = api_client.get(
        "/api/v1/auth/tokens/{}".format(token["id"]),
        headers={
            "Accept": "application/json",
            "Authorization": "Bearer {}".format(request_token),
        }
    )
    assert r.status_code == 404



from collections import OrderedDict
from typing import Dict
from uuid import UUID

from chaosplt_auth.model import AccessToken
from chaosplt_auth.schemas import AccessTokenSchema, NewAccessTokenSchema, \
    token_schema, new_token_schema
from flask import Flask


def test_parse_access_token_request(access_token_request: Dict[str, str]):
    token = new_token_schema.load(access_token_request)
    assert token == access_token_request


def test_access_token_schema_serialization(api: Flask, user_id: UUID,
                                           access_token_id: UUID):
    info = dict(
        id=str(access_token_id),
        name="my token",
        user_id=user_id,
        access_token="access-123456",
        refresh_token="refresh-123456",
    )
    token = AccessToken(**info)

    expected_info = OrderedDict(
        id=str(access_token_id),
        user_id=str(user_id),
        name="my token",
        access_token="access-123456",
        refresh_token="refresh-123456",
        url='http://localhost/api/v1/auth/tokens/{}'.format(access_token_id),
        links={
            'self': '/api/v1/auth/tokens/{}'.format(access_token_id),
        }
    )

    with api.test_request_context():
        token = token_schema.dump(token)
        assert token == expected_info


def test_parse_access_token(api: Flask, user_id: UUID,
                            access_token_id: UUID):
    info = dict(
        name="my token",
        id=access_token_id,
        user_id=user_id,
        access_token="access-123456",
        refresh_token="refresh-123456",
    )

    with api.test_request_context():
        token = token_schema.load(info)
        assert token == AccessToken(**info)



import uuid
from uuid import UUID

from chaosplt_auth.views.web.backend import OAuthBackend


def test_get_oauth_token_before_create(oauth_backend: OAuthBackend,
                                       user_id: UUID):
    token = oauth_backend.get_oauth_token(user_id)
    assert token == None


def test_get_oauth_token_after_create(oauth_backend: OAuthBackend,
                                      user_id: UUID):
    token = oauth_backend.get_oauth_token(user_id)
    assert token == None

    new_token = oauth_backend.create_oauth_token(
        user_id, "github", "12345", {"token": "12335"})

    token = oauth_backend.get_oauth_token(user_id)
    assert token == new_token


def test_delete_oauth_token_after_create(oauth_backend: OAuthBackend,
                                         user_id: UUID):
    token = oauth_backend.get_oauth_token(user_id)
    assert token == None

    new_token = oauth_backend.create_oauth_token(
        user_id, "github", uuid.uuid4().hex, {"token": "12335"})

    token = oauth_backend.get_oauth_token(user_id)
    assert token == new_token

    oauth_backend.delete_oauth_token(user_id)

    token = oauth_backend.get_oauth_token(user_id)
    assert token == None


def test_get_oauth_token_by_provider(oauth_backend: OAuthBackend,
                                     user_id: UUID):
    token = oauth_backend.get_oauth_token(user_id)
    assert token == None

    provider_id = uuid.uuid4().hex
    new_token = oauth_backend.create_oauth_token(
        user_id, "github", provider_id, {"token": "12335"})

    token = oauth_backend.get_oauth_token_by_provider("github", provider_id)
    assert token == new_token



def test_update_oauth_token(oauth_backend: OAuthBackend, user_id: UUID):
    token = oauth_backend.get_oauth_token(user_id)
    assert token == None

    token = oauth_backend.create_oauth_token(
        user_id, "github", uuid.uuid4().hex, {"token": "12335"})
    oauth_backend.set_oauth_token(user_id, {"token": "456789"})

    token = oauth_backend.get_oauth_token(user_id)
    assert token.token == {"token": "456789"}



import logging
from logging import StreamHandler
import os
import random
import string
from typing import Any, Dict
from unittest.mock import MagicMock
import uuid
from uuid import UUID

import cherrypy
from flask import Flask, request
from flask_caching import Cache
from flask_login import login_required, login_user, logout_user

import pytest

from chaosplt_auth.cache import setup_cache
from chaosplt_auth.log import http_requests_logger
from chaosplt_auth.views.api import create_api, cleanup_api, serve_api, \
    register_api
from chaosplt_auth.views.web.backend import OAuthBackend
from chaosplt_auth.views.web import create_app, cleanup_app, serve_app, \
    register_views
from chaosplt_auth.model import User
from chaosplt_relational_storage.db import RelationalStorage
from chaosplt_auth.service import Services, initialize_services
from chaosplt_auth.settings import load_settings
from chaosplt_auth.storage import initialize_storage, AuthStorage
from flask_jwt_extended import create_access_token, create_refresh_token

cur_dir = os.path.abspath(os.path.dirname(__file__))
fixtures_dir = os.path.join(cur_dir, "fixtures")
env_path = os.path.join(fixtures_dir, '.env')


@pytest.fixture
def config():
    return load_settings(env_path)


@pytest.fixture
def storage(config: Dict[str, Any]) -> AuthStorage:
    return initialize_storage(config)


@pytest.fixture
def stream_handler() -> StreamHandler:
    return StreamHandler()


@pytest.fixture
def account_service():
    return MagicMock()


@pytest.fixture
def services(config: Dict[str, Any], account_service) -> Services:
    svc = Services()
    svc.account = account_service
    initialize_services(svc, config)
    return svc


@pytest.fixture
def app(config: Dict[str, Any], services: Services, storage: AuthStorage,
        stream_handler: StreamHandler) -> Flask:
    application = create_app(config)
    cache = setup_cache(application)
    serve_app(
        application, cache, services, storage, config, "/auth",
        log_handler=stream_handler)
    return application


@pytest.fixture
def api(app: Flask, config: Dict[str, Any], services: Services,
        storage: AuthStorage, stream_handler: StreamHandler) -> Flask:
    application = create_api(config)
    cache = setup_cache(application)
    wsgiapp = http_requests_logger(application, stream_handler)
    cherrypy.tree.graft(wsgiapp, "/api/v1")
    serve_api(
        application, cache, services, storage, config, "/api/v1/auth",
        log_handler=stream_handler)
    return application


@pytest.fixture
def app_cache(app: Flask) -> Cache:
    return Cache(app, config=app.config)


@pytest.fixture
def api_cache(api: Flask) -> Cache:
    return Cache(api, config=api.config)


@pytest.fixture
def user_id() -> UUID:
    return uuid.uuid4() #UUID("1d77476b-f11d-433b-b7ee-bcbf25862051")


@pytest.fixture
def username() -> str:
    return "jane"


@pytest.fixture
def name() -> str:
    return "jane martin"


@pytest.fixture
def email() -> str:
    return "jane@world.com"


@pytest.fixture
def user(user_id: UUID, username: str, name: str, email: str):
    return User(user_id, username, name, email, True, True, False)


@pytest.fixture
def access_token_id() -> UUID:
    return UUID("86693512-82e9-4dec-8d54-53c312272300")


@pytest.fixture
def app_client(app: Flask, services: Services, username: str, name: str,
               email: str):
    app.config['TESTING'] = True
    app.testing = True

    with app.test_request_context():
        user = services.account.registration.create(
            username, name, email)
        login_user(user, remember=True)
        services.account.registration.get(user_id)

    with app.test_client() as client:
        yield client


@pytest.fixture
def api_client(api: Flask, services: Services, username: str, name: str,
               email: str):
    api.config['TESTING'] = True
    api.testing = True

    with api.test_request_context():
        user = services.account.registration.create(
            username, name, email)
        login_user(user, remember=True)
        services.account.registration.get(user_id)

    with api.test_client() as client:
        yield client


@pytest.fixture
def access_token_name() -> str:
    return uuid.uuid4().hex


@pytest.fixture
def access_token_request(user_id: UUID,
                         access_token_name: str) -> Dict[str, str]:
    return {
        "name": access_token_name,
        "user_id": str(user_id),
        "scopes": "read"
    }


@pytest.fixture
def request_token(app: Flask, api: Flask, user_id: UUID) -> str:
    with api.app_context():
        return create_access_token(str(user_id))


@pytest.fixture
def oauth_backend(services: Services, storage: AuthStorage, app_cache: Cache):
    return OAuthBackend(services, storage, app_cache)



from typing import Any, Dict, NoReturn

from chaosplt_relational_storage import get_storage, \
    configure_storage, release_storage
import pkg_resources

from .concrete import AccessTokenService, OAuthTokenService
from .interface import BaseAuthStorage

__all__ = ["initialize_storage", "shutdown_storage", "AuthStorage"]


class AuthStorage(BaseAuthStorage):
    def __init__(self, config: Dict[str, Any]):
        self.driver = get_storage(config)
        configure_storage(self.driver)

        access_token = AccessTokenService(self.driver)
        oauth_token = OAuthTokenService(self.driver)
        BaseAuthStorage.__init__(self, access_token, oauth_token)

    def release(self) -> NoReturn:
        """
        Release the storage resources.
        """
        release_storage(self.driver)


def initialize_storage(config: Dict[str, Any]) -> BaseAuthStorage:
    """
    Initialize the underlying authentication storage.

    If an installed package exposes the `chaosplatform.storage` class
    entrypoint for the `"auth"` group, it will be loaded. Otherwise, it'll be
    the default `AuthStorage` from this module.
    """
    for plugin in pkg_resources.iter_entry_points('chaosplatform.storage'):
        if plugin.name == "auth":
            service_class = plugin.load()
            return service_class(config)

    return AuthStorage(config)


def shutdown_storage(storage: BaseAuthStorage) -> NoReturn:
    """
    Release the storage resources by calling its `release` method.
    """
    storage.release()



from typing import Dict, NoReturn, Union
from uuid import UUID

from chaosplt_auth.model import AccessToken, OAuthToken
from chaosplt_relational_storage import RelationalStorage
from chaosplt_relational_storage.db import orm_session

from .interface import BaseAccessTokenService, BaseOAuthTokenService
from .model import AccessToken as AccessTokenModel, \
    OAuthToken as OAuthTokenModel


class AccessTokenService(BaseAccessTokenService):
    def __init__(self, driver: RelationalStorage):
        self.driver = driver

    def get(self, user_id: Union[UUID, str],
            token_id: Union[UUID, str]) -> AccessToken:
        with orm_session() as session:
            token = AccessTokenModel.load(user_id, token_id, session=session)
            if not token:
                return

            return AccessToken(
                name=token.name,
                id=token.id,
                user_id=token.user_id,
                access_token=token.access_token,
                refresh_token=token.refresh_token
            )

    def create(self, name: str, user_id: str, access_token: str,
               refresh_token: str) -> AccessToken:
        with orm_session() as session:
            token = AccessTokenModel.save(
                name, user_id, access_token, refresh_token, session=session)
            session.flush()

            return AccessToken(
                name=name,
                id=token.id,
                user_id=user_id,
                access_token=access_token,
                refresh_token=refresh_token
            )

    def get_by_user(self, user_id: Union[UUID, str]) -> Dict[str, AccessToken]:
        with orm_session() as session:
            result = {}
            tokens = AccessTokenModel.load_by_user(user_id, session=session)
            if tokens:
                for token in tokens:
                    result[token.name] = AccessToken(
                        name=token.name,
                        id=token.id,
                        user_id=token.user_id,
                        access_token=token.access_token,
                        refresh_token=token.refresh_token
                    )
            return result

    def delete(self, user_id: Union[UUID, str], token_id: Union[UUID, str]):
        with orm_session() as session:
            AccessTokenModel.delete(user_id, token_id, session=session)


class OAuthTokenService(BaseOAuthTokenService):
    def __init__(self, driver: RelationalStorage):
        self.driver = driver

    def get(self, user_id: Union[UUID, str],
            oauth_id: Union[UUID, str]) -> OAuthToken:
        with orm_session() as session:
            oauth = OAuthTokenModel.load(user_id, oauth_id, session=session)

            if oauth:
                return OAuthToken(
                    id=oauth.id,
                    user_id=oauth.user_id,
                    provider=oauth.provider,
                    provider_id=oauth.provider_id,
                    token=oauth.token
                )

    def get_by_provider(self, provider: str, provider_id: str) -> OAuthToken:
        with orm_session() as session:
            oauth = OAuthTokenModel.load_by_provider(
                provider, provider_id, session=session)

            if oauth:
                return OAuthToken(
                    id=oauth.id,
                    user_id=oauth.user_id,
                    provider=oauth.provider,
                    provider_id=oauth.provider_id,
                    token=oauth.token
                )

    def get_by_user(self, user_id: Union[UUID, str]) -> OAuthToken:
        with orm_session() as session:
            oauth = OAuthTokenModel.load_by_user(user_id, session=session)

            if oauth:
                return OAuthToken(
                    id=oauth.id,
                    user_id=oauth.user_id,
                    provider=oauth.provider,
                    provider_id=oauth.provider_id,
                    token=oauth.token
                )

    def create(self, user_id: Union[UUID, str], provider: str,
               provider_id: str, token: str) -> OAuthToken:
        with orm_session() as session:
            oauth = OAuthTokenModel.save(
                user_id, provider, provider_id, token, session=session)
            session.flush()

            return OAuthToken(
                id=oauth.id,
                user_id=oauth.user_id,
                provider=oauth.provider,
                provider_id=oauth.provider_id,
                token=oauth.token
            )

    def delete(self, user_id: Union[UUID, str],
               oauth_id: Union[UUID, str]) -> NoReturn:
        with orm_session() as session:
            OAuthTokenModel.delete(user_id, oauth_id, session=session)

    def delete_by_user(self, user_id: Union[UUID, str]) -> NoReturn:
        with orm_session() as session:
            OAuthTokenModel.delete_by_user(user_id, session=session)

    def set_for_user(self, user_id: Union[UUID, str], token: str) -> NoReturn:
        with orm_session() as session:
            OAuthTokenModel.update_by_user(user_id, token, session=session)



import pytest

from chaosplt_auth.storage.interface import BaseAccessTokenService, \
    BaseOAuthTokenService


def test_cannot_instanciate_acess_token_interface():
    try:
        BaseAccessTokenService()
    except TypeError as e:
        return
    else:
        pytest.fail("BaseAccessTokenService should remain abstract")


def test_cannot_instanciate_acess_token_interface():
    try:
        BaseOAuthTokenService()
    except TypeError as e:
        return
    else:
        pytest.fail("BaseOAuthTokenService should remain abstract")



# -*- coding: utf-8 -*-
from uuid import UUID

from flask import abort, Blueprint, jsonify, request
from flask_jwt_extended import create_access_token, create_refresh_token
from flask_login import current_user, login_required
from marshmallow import ValidationError

from chaosplt_auth.schemas import new_token_schema, token_schema

__all__ = ["api"]

api = Blueprint("token", __name__)


@api.route('access', methods=['GET', 'HEAD', 'POST', 'PUT', 'DELETE', 'PATCH'])
@login_required
def validate():
    """
    Endpoint used to validate the credentials from a user. If this function
    is ever called, then it means the credentials are indeed valid otherwise
    an error was triggered before.
    """
    return "", 200


@api.route('', methods=['POST'])
@login_required
def new():
    try:
        payload = new_token_schema.load(request.json)
    except ValidationError as err:
        return jsonify(err.messages), 422

    name = payload["name"]
    user_id = payload["user_id"]

    if str(current_user.id) != user_id:
        return jsonify({
            "message": "Cannot create a token for another user"
        }), 422

    user = request.services.account.registration.get(user_id)
    if not user:
        return jsonify({
            "message": "Unknown user"
        }), 422

    tokens = request.storage.access_token.get_by_user(user_id)
    if name in tokens:
        return jsonify({
            "message": "Name already used for another of your tokens"
        }), 409

    access_token = create_access_token(str(user_id))
    refresh_token = create_refresh_token(str(user_id))
    token = request.storage.access_token.create(
        name, user_id, access_token, refresh_token)
    return token_schema.jsonify(token), 201


@api.route('<uuid:token_id>', methods=['GET'])
@login_required
def get(token_id: UUID):
    token = request.storage.access_token.get(current_user.id, token_id)
    if not token:
        return abort(404)
    return token_schema.jsonify(token)


@api.route('<uuid:token_id>', methods=['DELETE'])
@login_required
def delete(token_id: UUID):
    request.storage.access_token.delete(current_user.id, token_id)
    return "", 204



from datetime import timedelta
from typing import Any, Dict, NoReturn, Union
from uuid import UUID

from flask.json import JSONEncoder
from flask_jwt_extended import JWTManager
from flask_jwt_extended.tokens import encode_refresh_token, encode_access_token

from chaosplt_grpc.auth.message import AccessToken
from chaosplt_grpc.auth.server import AuthService as GRPCAuthService

from .storage import AuthStorage

__all__ = ["AuthRPC"]


class NoAppJWTManager(JWTManager):
    def __init__(self, config: Dict[str, Any]):
        JWTManager.__init__(self, app=None)
        self.config = config
        self._encode_key_callback = self.encode_key_callback

    def encode_key_callback(self, identity: Union[UUID, str]):
        return self.config["secret_key"]

    def _user_identity_callback(self, identity: Union[UUID, str]):
        return identity

    def _create_refresh_token(self, identity: Union[UUID, str]):
        expires_delta = timedelta(
            seconds=int(self.config["refresh_token_expires"]))

        if self.config["user_claims_in_refresh_token"]:
            user_claims = self._user_claims_callback(identity)
        else:
            user_claims = None

        refresh_token = encode_refresh_token(
            identity=self._user_identity_callback(identity),
            secret=self._encode_key_callback(identity),
            algorithm=self.config["algorithm"],
            expires_delta=expires_delta,
            user_claims=user_claims,
            csrf=False,
            identity_claim_key=self.config["identity_claim_key"],
            user_claims_key=self.config["user_claims_key"],
            json_encoder=JSONEncoder
        )
        return refresh_token

    def _create_access_token(self, identity: Union[UUID, str]):
        expires_delta = timedelta(
            seconds=int(self.config["access_token_expires"]))

        access_token = encode_access_token(
            identity=self._user_identity_callback(identity),
            secret=self._encode_key_callback(identity),
            algorithm=self.config["algorithm"],
            expires_delta=expires_delta,
            fresh=False,
            user_claims=self._user_claims_callback(identity),
            csrf=False,
            identity_claim_key=self.config["identity_claim_key"],
            user_claims_key=self.config["user_claims_key"],
            json_encoder=JSONEncoder
        )
        return access_token


class AuthRPC(GRPCAuthService):
    def __init__(self, storage: AuthStorage, config: Dict[str, Any]):
        GRPCAuthService.__init__(self)
        self.storage = storage
        self.jwt = NoAppJWTManager(config.get("jwt"))

    def create_access_token(self, user_id: str, name: str) -> AccessToken:
        access_token = self.jwt._create_access_token(str(user_id))
        refresh_token = self.jwt._create_refresh_token(str(user_id))

        token = self.storage.access_token.create(
             name, user_id, access_token, refresh_token)
        return AccessToken(
            id=str(token.id),
            user_id=str(token.user_id),
            name=token.name,
            access_token=token.access_token,
            refresh_token=token.refresh_token
        )

    def delete_access_token(self, user_id: str, token_id: str) -> NoReturn:
        self.storage.access_token.delete(user_id, token_id)



import uuid

from chaosplt_auth.storage.concrete import AccessTokenService, OAuthTokenService
from chaosplt_auth.storage.model import AccessToken, OAuthToken
from chaosplt_relational_storage.db import orm_session, RelationalStorage


def test_save_access_token(storage: RelationalStorage):
    svc = AccessTokenService(storage)
    token = svc.create(
        name=uuid.uuid4().hex,
        user_id=uuid.uuid4(),
        access_token="at-123456",
        refresh_token="rt-123456",
    )
    assert uuid.UUID(hex=token.id.hex) == token.id


def test_get_access_token(storage: RelationalStorage):
    svc = AccessTokenService(storage)
    user_id = uuid.uuid4()

    token = svc.create(
        name=uuid.uuid4().hex,
        user_id=user_id,
        access_token="at-123456",
        refresh_token="rt-123456",
    )

    fetched_token = svc.get(user_id, token.id)
    assert token == fetched_token


def test_get_access_token_for_user(storage: RelationalStorage):
    svc = AccessTokenService(storage)
    user_id = uuid.uuid4()

    token = svc.create(
        name=uuid.uuid4().hex,
        user_id=user_id,
        access_token="at-123456",
        refresh_token="rt-123456",
    )

    fetched_token = svc.get_by_user(user_id)
    assert {token.name: token} == fetched_token


def test_delete_access_token(storage: RelationalStorage):
    svc = AccessTokenService(storage)
    user_id = uuid.uuid4()

    token = svc.create(
        name=uuid.uuid4().hex,
        user_id=user_id,
        access_token="at-123456",
        refresh_token="rt-123456",
    )

    fetched_token = svc.get_by_user(user_id)
    assert {token.name: token} == fetched_token

    svc.delete(user_id, token.id)
    
    fetched_token = svc.get_by_user(user_id)
    assert fetched_token == {}


def test_save_oauth2_token(storage: RelationalStorage):
    svc = OAuthTokenService(storage)

    token = svc.create(
        user_id=uuid.uuid4(),
        provider="github",
        provider_id=uuid.uuid4().hex,
        token={"token": "a-token"}
    )
    assert token.token == {"token": "a-token"}


def test_get_oauth2_token(storage: RelationalStorage):
    svc = OAuthTokenService(storage)
    user_id = uuid.uuid4()

    token = svc.create(
        user_id=user_id,
        provider="github",
        provider_id=uuid.uuid4().hex,
        token={"token": "a-token"}
    )
    fetched_token = svc.get(user_id, token.id)
    assert token == fetched_token


def test_cannot_get_oauth2_token_for_different_user(storage: RelationalStorage):
    svc = OAuthTokenService(storage)
    user_id = uuid.uuid4()

    token = svc.create(
        user_id=user_id,
        provider="github",
        provider_id=uuid.uuid4().hex,
        token={"token": "a-token"}
    )

    fetched_token = svc.get(user_id, token.id)
    assert fetched_token == token

    fetched_token = svc.get(uuid.uuid4(), token.id)
    assert fetched_token == None


def test_cannot_get_for_different_user(storage: RelationalStorage):
    svc = OAuthTokenService(storage)
    user_id = uuid.uuid4()

    token = svc.create(
        user_id=user_id,
        provider="github",
        provider_id=uuid.uuid4().hex,
        token={"token": "a-token"}
    )

    fetched_token = svc.get_by_user(user_id)
    assert fetched_token == token

    fetched_token = svc.get_by_user(uuid.uuid4())
    assert fetched_token == None


def test_get_oauth2_token_by_provider(storage: RelationalStorage):
    svc = OAuthTokenService(storage)
    user_id = uuid.uuid4()
    provider_id= uuid.uuid4().hex

    token = svc.create(
        user_id=user_id,
        provider="github",
        provider_id=provider_id,
        token={"token": "a-token"}
    )

    fetched_token = svc.get_by_provider("github", provider_id)
    assert fetched_token == token

    fetched_token = svc.get_by_provider("bitbucket", provider_id)
    assert fetched_token == None


def test_delete_oauth2_token(storage: RelationalStorage):
    svc = OAuthTokenService(storage)
    user_id = uuid.uuid4()
    provider_id= uuid.uuid4().hex

    token = svc.create(
        user_id=user_id,
        provider="github",
        provider_id=provider_id,
        token={"token": "a-token"}
    )

    fetched_token = svc.get(user_id, token.id)
    assert token == fetched_token

    svc.delete(user_id, token.id)
    fetched_token = svc.get(user_id, token.id)
    assert fetched_token == None


def test_delete_oauth2_token_by_user(storage: RelationalStorage):
    svc = OAuthTokenService(storage)
    user_id = uuid.uuid4()
    provider_id= uuid.uuid4().hex

    token = svc.create(
        user_id=user_id,
        provider="github",
        provider_id=provider_id,
        token={"token": "a-token"}
    )

    fetched_token = svc.get(user_id, token.id)
    assert token == fetched_token

    svc.delete_by_user(user_id)
    fetched_token = svc.get(user_id, token.id)
    assert fetched_token == None


def test_update_oauth2_token(storage: RelationalStorage):
    svc = OAuthTokenService(storage)
    user_id = uuid.uuid4()
    provider_id= uuid.uuid4().hex

    token = svc.create(
        user_id=user_id,
        provider="github",
        provider_id=provider_id,
        token={"token": "a-token"}
    )

    svc.set_for_user(user_id, {"token": "b-token"})

    fetched_token = svc.get(user_id, token.id)
    assert {"token": "a-token"} != fetched_token.token
    assert {"token": "b-token"} == fetched_token.token



from typing import Any, Dict

import attr

from .account import AccountService

__all__ = ["initialize_services", "shutdown_services", "Services"]


@attr.s
class Services:
    account: AccountService = attr.ib(default=None)


def initialize_services(services: Services, config: Dict[str, Any]):
    """
    Initialize all services that the authentication service needs to work:

    * account service
    """
    if not services.account:
        services.account = AccountService(config)


def shutdown_services(services: Services):
    """
    Clear out the services used by the authentication service.
    """
    pass



import uuid

from chaosplt_auth.storage import AuthStorage
from chaosplt_auth.storage.model import AccessToken, OAuthToken
from chaosplt_relational_storage.db import orm_session


def test_save_access_token(storage: AuthStorage):
    with orm_session() as session:
        token = AccessToken.save(
            name=uuid.uuid4().hex,
            user_id=uuid.uuid4(),
            access_token="at-123456",
            refresh_token="rt-123456",
            session=session
        )
        session.commit()
        assert uuid.UUID(hex=token.id.hex) == token.id


def test_get_access_token(storage: AuthStorage):
    with orm_session() as session:
        user_id = uuid.uuid4()

        token = AccessToken.save(
            name=uuid.uuid4().hex,
            user_id=user_id,
            access_token="at-123456",
            refresh_token="rt-123456",
            session=session
        )
        session.commit()

        fetched_token = AccessToken.load(user_id, token.id, session)
        assert token == fetched_token


def test_get_access_token_for_user(storage: AuthStorage):
    with orm_session() as session:
        user_id = uuid.uuid4()

        token = AccessToken.save(
            name=uuid.uuid4().hex,
            user_id=user_id,
            access_token="at-123456",
            refresh_token="rt-123456",
            session=session
        )
        session.commit()

        fetched_tokens = AccessToken.load_by_user(user_id, session)
        assert fetched_tokens == [token]

        fetched_tokens = AccessToken.load_by_user(uuid.uuid4(), session)
        assert fetched_tokens == []


def test_delete_access_token(storage: AuthStorage):
    with orm_session() as session:
        user_id = uuid.uuid4()

        token = AccessToken.save(
            name=uuid.uuid4().hex,
            user_id=user_id,
            access_token="at-123456",
            refresh_token="rt-123456",
            session=session
        )
        session.commit()

        fetched_token = AccessToken.load(user_id, token.id, session)
        assert token == fetched_token

        AccessToken.delete(user_id, token.id, session)
        fetched_token = AccessToken.load(user_id, token.id, session)
        assert fetched_token == None


def test_save_oauth2_token(storage: AuthStorage):
    with orm_session() as session:
        token = OAuthToken.save(
            user_id=uuid.uuid4(),
            provider="github",
            provider_id=uuid.uuid4().hex,
            token={"token": "a-token"},
            session=session
        )
        session.commit()
        assert token.token == {"token": "a-token"}


def test_get_oauth2_token(storage: AuthStorage):
    with orm_session() as session:
        user_id = uuid.uuid4()

        token = OAuthToken.save(
            user_id=user_id,
            provider="github",
            provider_id=uuid.uuid4().hex,
            token={"token": "a-token"},
            session=session
        )
        session.commit()

        fetched_token = OAuthToken.load(user_id, token.id, session)
        assert token == fetched_token


def test_cannot_get_oauth2_token_for_different_user(storage: AuthStorage):
    with orm_session() as session:
        user_id = uuid.uuid4()

        token = OAuthToken.save(
            user_id=user_id,
            provider="github",
            provider_id=uuid.uuid4().hex,
            token={"token": "a-token"},
            session=session
        )
        session.commit()

        fetched_token = OAuthToken.load(user_id, token.id, session)
        assert fetched_token == token

        fetched_token = OAuthToken.load(uuid.uuid4(), token.id, session)
        assert fetched_token == None


def test_get_oauth2_token_for_user(storage: AuthStorage):
    with orm_session() as session:
        user_id = uuid.uuid4()

        token = OAuthToken.save(
            user_id=user_id,
            provider="github",
            provider_id=uuid.uuid4().hex,
            token={"token": "a-token"},
            session=session
        )
        session.commit()

        fetched_token = OAuthToken.load_by_user(user_id, session)
        assert fetched_token == token

        fetched_token = OAuthToken.load_by_user(uuid.uuid4(), session)
        assert fetched_token == None


def test_get_oauth2_token_by_provider(storage: AuthStorage):
    with orm_session() as session:
        user_id = uuid.uuid4()
        provider_id = uuid.uuid4().hex

        token = OAuthToken.save(
            user_id=user_id,
            provider="github",
            provider_id=provider_id,
            token={"token": "a-token"},
            session=session
        )
        session.commit()

        fetched_token = OAuthToken.load_by_provider(
            "github", provider_id, session)
        assert fetched_token == token

        fetched_token = OAuthToken.load_by_provider(
            "bitbucket", provider_id, session)
        assert fetched_token == None


def test_delete_oauth2_token(storage: AuthStorage):
    with orm_session() as session:
        user_id = uuid.uuid4()

        token = OAuthToken.save(
            user_id=user_id,
            provider="github",
            provider_id=uuid.uuid4().hex,
            token={"token": "a-token"},
            session=session
        )
        session.commit()

        fetched_token = OAuthToken.load(user_id, token.id, session)
        assert token == fetched_token

        OAuthToken.delete(user_id, token.id, session)
        fetched_token = OAuthToken.load(user_id, token.id, session)
        assert fetched_token == None


def test_delete_oauth2_token_by_user(storage: AuthStorage):
    with orm_session() as session:
        user_id = uuid.uuid4()

        token = OAuthToken.save(
            user_id=user_id,
            provider="github",
            provider_id=uuid.uuid4().hex,
            token={"token": "a-token"},
            session=session
        )
        session.commit()

        fetched_token = OAuthToken.load(user_id, token.id, session)
        assert token == fetched_token

        OAuthToken.delete_by_user(user_id, session)
        fetched_token = OAuthToken.load(user_id, token.id, session)
        assert fetched_token == None


def test_update_oauth2_token(storage: AuthStorage):
    with orm_session() as session:
        user_id = uuid.uuid4()

        token = OAuthToken.save(
            user_id=user_id,
            provider="github",
            provider_id=uuid.uuid4().hex,
            token={"token": "a-token"},
            session=session
        )
        session.commit()

        OAuthToken.update_by_user(user_id, {"token": "b-token"}, session)

        fetched_token = OAuthToken.load(user_id, token.id, session)
        assert {"token": "a-token"} != fetched_token.token
        assert {"token": "b-token"} == fetched_token.token



from typing import Any, Dict
from uuid import UUID

import jwt

from chaosplt_auth.rpc import AuthRPC
from chaosplt_auth.storage import AuthStorage


def test_create_auth_rpc_service(config: Dict[str, Any], storage: AuthStorage,
                                 user_id: UUID, name: str):
    rpc = AuthRPC(storage, config)
    token = rpc.create_access_token(user_id, name)

    assert token.name == name
    assert token.user_id == str(user_id)
    assert token.id is not None
    assert token.access_token is not None
    assert token.refresh_token is not None

    access_token_claim = jwt.decode(
        token.access_token, config["jwt"]["secret_key"],
        algorithms=[config["jwt"]["algorithm"]])
    assert access_token_claim[config["jwt"]["identity_claim_key"]] == str(user_id)

    refresh_token_claim = jwt.decode(
        token.access_token, config["jwt"]["secret_key"],
        algorithms=[config["jwt"]["algorithm"]])
    assert refresh_token_claim[config["jwt"]["identity_claim_key"]] == str(user_id)


def test_delete_auth_rpc_service(config: Dict[str, Any], storage: AuthStorage,
                                 user_id: UUID, name: str):
    rpc = AuthRPC(storage, config)
    token = rpc.create_access_token(user_id, name)
    assert token.name == name
    assert token.user_id == str(user_id)
    assert token.id is not None

    access_token = storage.access_token.get(user_id, token.id)
    assert access_token is not None

    rpc.delete_access_token(user_id, token.id)
    access_token = storage.access_token.get(user_id, token.id)
    assert access_token is None



# -*- coding: utf-8 -*-
from flask import Blueprint
from flask_dance.contrib.github import make_github_blueprint
from flask_login import login_required

__all__ = ["view", "github_view"]

view = Blueprint("auth", __name__)
github_view = make_github_blueprint()


@view.route('')
@login_required
def index():
    return "", 200
